###############################################################################
# Copyright (C) 2014 by gempa GmbH                                            #
#                                                                             #
# All Rights Reserved.                                                        #
#                                                                             #
# NOTICE: All information contained herein is, and remains                    #
# the property of gempa GmbH and its suppliers, if any. The intellectual      #
# and technical concepts contained herein are proprietary to gempa GmbH       #
# and its suppliers.                                                          #
# Dissemination of this information or reproduction of this material          #
# is strictly forbidden unless prior written permission is obtained           #
# from gempa GmbH.                                                            #
#                                                                             #
# Author: Stephan Herrnkind                                                   #
# Email: herrnkind@gempa.de                                                   #
###############################################################################

import os
import sys


# Joins given arguments into an url. Trailing and leading slashes are
# stripped for each argument.
def urljoin(*args):
    return "/".join(map(lambda x: str(x).strip("/"), args))


LICENSES = "licenses"
BASE_URL = "https://data.gempa.de/"
BASE_PUBLIC_URL = urljoin(BASE_URL, "packages", "Public")

RENAME_SEEDLINK_BUFFER = "gempa-rename-seedlink-buffer.sh"
CHECK_SCCONFIG = "gempa-checkSCconfig.py"
SEISCOMP_TOOLS = "seiscomp-tools"

PUBLIC_PACKAGES = [SEISCOMP_TOOLS]

DOC_URL = "https://docs.gempa.de/"

NIGHTLY = "nightly"
SC = "seiscomp"
SC3 = "seiscomp3"

DEFAULT_RELEASE = "6"
DEFAULT_ARCH = "x86_64"

ARCHIVE = "archive"

NO_ARCH = "noarch"
ANY = "any"
EXTEND_DIR = "extend"
ANY_NOARCH_DIR = os.path.join(ANY, NO_ARCH)
CUSTOM_INSTALL_PATH = os.path.join("libs", "cache", "custominstall")

FORMAT = {"0.2": "1.4.0"}

GSM_INFO_URL = urljoin(BASE_URL, "gsminfo/gsm.info")
GSM_INFO_OBSOLETE_PACKAGES_URL = urljoin(BASE_URL, "gsminfo/")

GREENSFUNCTIONS = "greensfunctions"

LICENSE_INFO_DAYS = 30

FILE_ENCODING = "utf8"

LINE_WIDTH = 80

DEFAULT_TIMEOUT = 180

BAR_RIGHT_LENGHT = 61


class PT:
    def __init__(self):
        self.APP = None
        self.DATA = None
        self.CONFIG = None
        self.GSM = None


PT.APP = "app"
PT.DATA = "data"
PT.CONFIG = "config"
PT.GSM = "gsm"

OS_MAPPING = {"centos": "rhel", "rocky": "rhel", "raspbian": "debian"}

LICENSE_MAPPING = {
    "caps-server": "caps",
    "gaps-server": "gaps",
    "shard-server": "shard",
}

DEFAULT_TIME_FORMAT = "%Y-%m-%d"
DEFAULT_DATE_FORMAT = "%Y-%m-%d %H:%M:%S"
DEFAULT_UPDATE_DATE_FORMAT = "%Y.%j"

INSTALL_DESC = "desc"
INSTALL_FILES = "files"

HOLD_FILE = ".hold"
MBTILES_EXTENSION = ".mbtiles"
GRD_EXTENSION = ".grd"
CRT_EXTENSION = ".crt"
DSC_EXTENSION = ".dsc"
ZIP_EXTENSION = ".zip"
TARGZ_EXTENSION = ".tar.gz"
TAR_EXTENSION = ".tar"
DB_EXTENSION = ".db"
CORE_DB = f"core{DB_EXTENSION}"

LICENSE_TXT_EXTENSION = "-license.txt"
CHANGELOG = "CHANGELOG"

OK = "[✓]"
FAILED = "[✗]"

try:
    OK.encode(sys.stdout.encoding)
    FAILED.encode(sys.stdout.encoding)
except UnicodeEncodeError:
    OK = "[O]"
    FAILED = "[X]"

ACCESS = "ACCESS"

GET = "GET"
GET_OK = f"{GET}      {OK}"
GET_FAILED = f"{GET}      {FAILED}"

INSTALL = "INSTALL"
INSTALL_OK = f"{INSTALL}  {OK}"
INSTALL_FAILED = f"{INSTALL}  {FAILED}"

CHECKSUM = "CHECKSUM"
CHECKSUM_OK = f"{CHECKSUM} {OK}"
CHECKSUM_FAILED = f"{CHECKSUM} {FAILED}"

CHECKING = "CHECKING"
CHECKING_OK = f"{CHECKING} {OK}"
CHECKING_FAILED = f"{CHECKING} {FAILED}"

REMOVE = "REMOVE"
REMOVE_OK = f"{REMOVE}   {OK}"
REMOVE_FAILED = f"{REMOVE}   {FAILED}"

MAX_LEN_PACKAGE_FULL_NAME = 30
