############################################################################
# Copyright (C) 2019 by gempa GmbH                                         #
#                                                                          #
# All Rights Reserved.                                                     #
#                                                                          #
# NOTICE: All information contained herein is, and remains                 #
# the property of gempa GmbH and its suppliers, if any. The intellectual   #
# and technical concepts contained herein are proprietary to gempa GmbH    #
# and its suppliers.                                                       #
# Dissemination of this information or reproduction of this material       #
# is strictly forbidden unless prior written permission is obtained        #
# from gempa GmbH.                                                         #
#                                                                          #
#  Author: Ralph Henneberger, Enrico Ellguth                               #
#  Email: ralix@gempa.de, enrico.ellguth@gempa.de                          #
############################################################################

import glob
import os
import re
import sys

import natsort

from libs import log


def getFilesList(path):
    try:
        return natsort.natsorted(
            glob.glob(os.path.join(path, "*.so.*.*.*")), reverse=True
        )
    except OSError as err:
        print(err, file=sys.stderr)

    return None


def getLinkList(fileNames):
    linkMap = {}
    linkMap_so = {}

    for fileName in fileNames:
        match = re.search(r"(.+)\.so\.(\d).(\d).(\d)", fileName)
        if not match:
            continue

        name = match.group(1)
        major = match.group(2)

        extendFileName = os.path.basename(name) + ".so." + major
        if not extendFileName in linkMap.values():
            linkMap.update(
                {os.path.basename(fileName): os.path.basename(extendFileName)}
            )

    for src, dst in linkMap.items():
        match = re.search(r"(.+)\.so\.(\d)", src)
        basename = match.group(1) + ".so"
        if not basename in linkMap_so.values():
            linkMap_so.update({dst: basename})

    for src, dst in linkMap_so.items():
        linkMap.update({src: dst})

    return linkMap


def createSymLinks(linkMap, path):
    if not os.path.isdir(path):
        return True

    try:
        os.chdir(os.path.join(os.getcwd(), path))
    except OSError as err:
        log.logError(f"Can't create library links: {err}")
        return False

    for src, dst in linkMap.items():
        try:
            os.remove(dst)
        except OSError as err:
            if os.path.isfile(dst):
                log.logError(f"Can't create library links: {err}")
                return False
        try:
            os.symlink(src, dst)
        except OSError as err:
            log.logError(f"Can't create library links: {err}")
            return False
    return True


def main():
    if len(sys.argv) != 2:
        print("\nmakelinks.py - Repairs broken links\n")
        print(f"Usage: {sys.argv[0]} <directory>")
        return False

    fileList = getFilesList(sys.argv[1])
    linkList = getLinkList(fileList)
    return not createSymLinks(linkList, sys.argv[1])


if __name__ == "__main__":
    sys.exit(main())
